#ifndef	_ST2_RESOURCE_H_
#define _ST2_RESOURCE_H_
#ifndef lint
static char rcsid_st2_resource_h[] = "\
@(#)  $Header: st2_resource.h,v 1.10 91/11/26 23:12:48 clynn Rel $  \n";
/*-------------------------------------------------------------------*
 |       Copyright (c) 1991 by BBN Systems and Technologies,         |
 |            A Division of Bolt Beranek and Newman Inc.             |
 |                                                                   |
 |  Permission to use, copy, modify, distribute, and sell this       |
 |  software and its documentation for any purpose is hereby         |
 |  granted without fee, provided that the above copyright notice    |
 |  and this permission appear in all copies and in supporting       |
 |  documentation, and that the name of Bolt Beranek and Newman      |
 |  Inc.  not be used in advertising or publicity pertaining to      |
 |  distribution of the software without specific, written prior     |
 |  permission. BBN makes no representations about the suitability   |
 |  of this software for any purposes.  It is provided "AS IS"       |
 |  without express or implied warranties.                           |
 *-------------------------------------------------------------------*/
#endif lint


/*
M* st2_resource.h  Definitions for ST-II Resource Management.
M*
 */

/*
m* Status:
m*   Features:
m*   Untested Features:
m*   Restrictions/Bugs:
m*   Things to do:
m*
 */


/* Module Revision History
 *
 *	bugid	9,0
 * $Log:	st2_resource.h,v $
 * Revision 1.10  91/11/26  23:12:48  clynn
 * Fixed typo.
 * 
 * Revision 1.9  91/11/04  09:23:00  clynn
 * Updated for Public Domain Release.  Major changes: addition of Source
 * Routing, IP Encapsulation, HELLO protocol between neighbors, tracking
 * of neighbors, detection of component or agent failures & notification
 * to applications.  More consistant naming and format, including making
 * all external names begin with "st2_".  Moved some routines and data
 * structures to reduce external references.
 * 
 * Minor documentation changes.
 * 
 * Revision 1.8  91/05/28  16:17:20  clynn
 * New features: Add new targets from Application layer, UserData support,
 * basic bandwidth reservation for point-to-point links, more complete
 * state tables, added pcode parameter to InitaFlowSpec3, extended packet
 * buffer abstraction, added network interface abstraction; ststat utility.
 * Bug fixes: ADDR IN USE problem, data send problem, causes of some
 * crashes, cleanup of protocol control blocks.
 * Work around: DARTNET receive memory leak.
 * Eliminated several small modules to reduce globals; adeded Makefile.
 * 
 * Revision 1.1  91/03/15  18:33:08  clynn
 * Initial revision
 */


/* Structure to hold information about resource requests for a requestor of
 * services from a network.
 */

struct aRsrcClnt {
    struct aRsrcClnt	*nextp;		/* Ptr to next client on this interface */
			/* ??? pending/multiple requests for single client */

    struct aRsrcId	 handle;	/* Resource id, given to client */

    unsigned long	 buf,		/* Resources allocated to client */
			 bw_clnt,
			 bw_local,
			 cpu_clnt,
			 cpu_local;

	/* Outstanding request (?s) for client */
/*  current request
    callback function
    callback argument
	struct NextHop *
    pending list
    lower layer, handle
    lower layer, ...
 */
};

/* ------------------------------------------------------------------------ */

/* Structure for managing buffer resources. */

struct aRsrcBuf {			/* # of buffers ... */
    unsigned long	 cnfg,		/* ... configured as available */
			 alloc;		/* ... currently allocated */
};

/* ------------------------------------------------------------------------ */

/* Structure for managing CPU resource. */

struct aRsrcCPU {			/* Fractional seconds ... */
    unsigned long	 cnfg,		/* ... configured as available */
			 alloc;		/* ... currently allocated */
};


/* Logical extension to OS native network interface structure
 * to support resource management by the network.
 */

struct aRsrcNet {
    struct aRsrcNet	*nextp;		/* ptr to next extension block */
    struct aNetIf	*gifcp;		/* ptr to generic interface this extends */

					/* Routines to ... */
    enum ST2Errors	(*if_rsrcgetid)(), /* ... get a resource id */
			(*if_rsrcrelid)(), /* ... release a resource id */
			(*if_rsrcprobe)(), /* ... probe available resources */
			(*if_rsrcalloc)(), /* ... allocate resources */
			(*if_rsrcrelse)(); /* ... release resources */

    struct aRsrcClnt	*clientsp;	/* ptr to clients of network */

    unsigned long	 flags;		/* Services provided by network */

# define NetCanMcast	0x01
# define NetMayMcast	0x02
# define NetCanBcast	0x04
# define NetMayBcast	0x08
# define NetIsPt2Pt	0x10

    struct aRsrcBuf	*bufp;		/* Ptr to buffer resources */
    struct aRsrcCPU	*cpup;		/* Ptr to CPU resources */
    unsigned long	 ber,		/* Bit Error Rate, negative of
					   exponent of ten ( > 0 ) */

					/* On-the-wire fractional secs ... */
			 bw_cnfg,	/* ... available, i.e., bandwidth */
			 bw_byte,	/* ... required to send a byte */
			 bw_pkt,	/* ... required to send a packet, incl.
					       gap, leader, header, trailer */
			 bw_alloc,	/* ... allocated bandwidth */

		/* ??? Need units - milli/micro/nano cents/dollars/pounds/DM */
			 cost,		/* Fixed cost per client */
			 cost_byte,	/* Per byte cost */
			 cost_msec,	/* per millisecond cost */
			 cost_pkt,	/* Per packet cost */

					/* Fractional seconds of CPU ... */
			 cpu_in_byte,	/* ... to receive a byte */
			 cpu_in_pkt,	/* ... to receive a packet */

			 cpu_out_byte,	/* ... to send a byte */
			 cpu_out_pkt,	/* ... to send a packet */

			 droprate,	/* Drop rate, fraction of packets that
					   will be dropped */
					/* ??? need per pkt & per byte? */

					/* Fractional seconds ... */
			 dly_in,	/* ... to receive a packet */
			 dly_in_var,

			 dly_prop,	/* ... of "fixed" propagation delay,
					       (else see aNeighbor delay) */
					/*     ???need >= 1 second */

			/* ??? array by class, etc., ??? need >= 1 second */
			 dly_que,	/* ... of queueing delay, estimate */
			 dly_que_var,
					/* ??? need per pkt & per byte? */
			 dly_out,	/* ... to transmit a packet */
			 dly_out_var,

			 lclhdrlen;	/* Max length of network-layer header,
					   so clients can leave room for it */
};
#endif	_ST2_RESOURCE_H_

/*
 Local Variables:
 c-indent-level: 4
 c-brace-offset: -4
 c-brace-imaginary-offset: 0
 c-continued-statement-offset: 4
 comment-column: 40
 End:
*/
