#ifndef lint
static char rcsid_st2_routing_c[] = "\
@(#)  $Header: st2_routing.c,v 1.10 91/11/26 23:16:54 clynn Rel $  \n";
/*-------------------------------------------------------------------*
 |       Copyright (c) 1991BBN Systems and Technologies,         |
 |            A Division of Bolt Beranek and Newman Inc.             |
 |                                                                   |
 |  Permission to use, copy, modify, distribute, and sell this       |
 |  software and its documentation for any purpose is hereby         |
 |  granted without fee, provided that the above copyright notice    |
 |  and this permission appear in all copies and in supporting       |
 |  documentation, and that the name of Bolt Beranek and Newman      |
 |  Inc.  not be used in advertising or publicity pertaining to      |
 |  distribution of the software without specific, written prior     |
 |  permission. BBN makes no representations about the suitability   |
 |  of this software for any purposes.  It is provided "AS IS"       |
 |  without express or implied warranties.                           |
 *-------------------------------------------------------------------*/
#endif lint


/*
M* st2_routing.c  Routines to Provide Routing Services to ST-II.
M*
m* Overview
m*
m*	ST-II assumes that it has access to a routing table which can
m*	(synchronously) provide one or more routes to a given
m*	destination.  st2_FindRoutes can thus build a matrix
m*	containing all routes from the current ST Agent to all of a
m*	stream's targets.  An routing optimizer (RouteOptimize)
m*	may then be called to "optimize" the routes by ordering the
m*	routing choices for each target.  Thus, for example, if two
m*	targets may be reached by the same next-hop, placing the route
m*	to that hop first would cause data for those targets to be
m*	sent via the same next-hop (allowing sharing of the
m*	bandwidth).  Similiarly, if two targets may be reached via the
m*	same network interface and that network supports multicast, a
m*	multicast group can be created so that a single copy of the
m*	data can be sent to the targets.
m*
m*	If the resource management routines are not able to obtain the
m*	Quality of Service specified in the FLowSpec, they return an
m*	error so the affected target's next routing choice will be
m*	tried.  If no route has the required resources, a routing
m*	failure is returned to the previous-hop so that it can proceed
m*	to its next routing choice.
m*
m*	ST-II assumes that the "routing tables" contain several pieces
m*	of information.  Included are the network interface to be used
m*	and whether the route is an ST route or an IP route.  ST
m*	routes include the IP Address of the next-hop ST agnet,
m*	whether IP Encapsulation is required and optional ST Source
m*	Routing Parameters.  IP routes include the IP address of the
m*	IP Entity toward the target, the IP TTL and TOS which should
m*	be used, and an optional IP Source Route Option.  This
m*	information is passed to ST-II for each target via an array of
m*	aRouteChoice structures.
m*
m* ??? This currently assumes that every reachable agent supports ST-II...
m*
 */

/*
m* Status:
m*   Features:
m*	IP Encapsulation.
m*	Source Routing.
m*   Untested Features:
m*	Multiple routing choices.
m*   Restrictions/Bugs:
m*	Assunes everything is ST capable & one hop away if not source routed.
m*   Things to do:
m*	Optimization.
m*	ST Routing table.
m*
 */


/* Module Revision History
 *
 * $Log:	st2_routing.c,v $
 * Revision 1.10  91/11/26  23:16:54  clynn
 * *** empty log message ***
 * 
 * Revision 1.9  91/11/04  09:23:08  clynn
 * Updated for Public Domain Release.  Major changes: addition of Source
 * Routing, IP Encapsulation, HELLO protocol between neighbors, tracking
 * of neighbors, detection of component or agent failures & notification
 * to applications.  More consistant naming and format, including making
 * all external names begin with "st2_".  Moved some routines and data
 * structures to reduce external references.
 * 
 * Added IP Encapsulation & Source Routing support.
 * 
 * Revision 1.8  91/05/28  16:18:02  clynn
 * New features: Add new targets from Application layer, UserData support,
 * basic bandwidth reservation for point-to-point links, more complete
 * state tables, added pcode parameter to InitaFlowSpec3, extended packet
 * buffer abstraction, added network interface abstraction; ststat utility.
 * Bug fixes: ADDR IN USE problem, data send problem, causes of some
 * crashes, cleanup of protocol control blocks.
 * Work around: DARTNET receive memory leak.
 * Eliminated several small modules to reduce globals; adeded Makefile.
 * 
 * Revision 1.1  91/03/15  18:33:29  clynn
 * Initial revision
 */

#define _FILE_ Strrut			/* Module bugid a,3 */


/* External definitions */

#include <sys/types.h>		/* for netinet/in.h */
#include <sys/socket.h>		/* sockaddr for net/if.h */
#include <net/if.h>		/* ifnet, ifaddr */
#include <netinet/in.h>		/* before st2.h -> st2_api.h; IPPROTO_*,
				   in_addr */

#include "st2_api.h"		/* Global Definitions, before st2.h */
#include "st2.h"		/* ST-II Implementation definitions,
				 * includes st2_api.h, before st2_cmp.h */
#include "st2_routing.h"


/* Local routines */

INETADDR		 st2_LocalIPAdrOfGifc (/* gifcp */);
void			 st2_FindRoutes (/* routeto, nextst, arclp, flags */);

#ifdef NOTYET
static void		 RouteOptimize (/* cp */);
					/* Optimizer */
void		       (*st2_RouteOptimizep) () = RouteOptimize;
#else					/* No optimizer */
void		       (*st2_RouteOptimizep) () = (void (*)()) NULL;
#endif !NOTYET

/*
S* st2_LocalIPAdrOfGifc ( gifcp )
 */

  INETADDR				/* Next bugid 0xa0102 */
st2_LocalIPAdrOfGifc( gifcp )		/* from HandleConnect NbrCreate st2_FindRoutes */
					/* ??? INLINE/MACRO */
  struct aNetIf		*gifcp;
{
    struct ifaddr	*ifap;
    struct ifnet	*ifp = AdrOfIfc (gifcp);


			/* ??? which destination to get best interface */
			/* ??? can an interface have multiple addresses? */
    for ( ifap = ifp->if_addrlist ; ifap NE (struct ifaddr *) NULL
	 ; ifap = ifap->ifa_next )
    {
	if ( ifap->ifa_addr.sa_family EQ AF_INET ) /* what if multiple??? */
	{
	    return ( (INETADDR) ((struct sockaddr_in *)
				 &( ifap->ifa_addr ))->sin_addr.s_addr );
	}
    }

    BUGSTOP (,ifp,0xa0101);
    return ( (INETADDR) INADDR_ANY );
}

/*
S* RouteOptimize ( cp )
s*
s*	Prioritize route choices for all targets which are not yet prioritized.
s*	The prioritization may take advantage of previously selected networks
s*	and/or next hops.
s*
 */
#ifdef NOTYET

  static void				/* Next bugid 0xa0200 */
RouteOptimize( cp )			/* from */
  struct aST2pcb	*cp;		/* ARGSUSED */
{
    return;
}
#endif NOTYET

/*
S* st2_FindRoutes ( routeto, nextst, arclp, flags )
s*
s*	Routine to find potential list of routes toward a specified target.
s*	Input:	routetoIPAdr, array for results, iphop/direct/no socket flags
s*	Output:	array entries (0 or more)
s*		nxtIPAdr
s*		IPencapsulate, TOS, TTL
s*		rtblsrlen, rtblsrp
s*
 */

  void					/* Next bugid 0xa0303 */
st2_FindRoutes( routeto, nextst, arclp, flags ) /* from HandleConnect NbrCreate st2_PCBUpdate */
  INETADDR		 routeto,	/* Destination IPAdr */
			 nextst;	/* Next ST Agent */
  struct aRouteChoiceList *arclp;	/* Exp array for results */
  int			 flags;		/* FRF_USEIP, FRF_DIRECT, FRF_NOSKT */
{
    INETADDR		 localIPAdr;
    struct aRouteChoice	*arcp = &( arclp->RouteChoices[0] );
    struct route	 path;		/* ??? BSDism */
    struct rtentry	*rep;
    struct sockaddr_in	*sip;


    /* Initialize RouteChoice list, BSD only has one route ... */

    arclp->nxtfree = 0;	/* Reinitialize routing choices */
    Bzero (arcp,sizeof (struct aRouteChoice));


    /* ??? Use BSD kernel route tables via inet family to get route */
    /* ??? try ST table then IP */

    Bzero (&( path ),sizeof (path));
    sip = (struct sockaddr_in *) &( path.ro_dst );
    /* (flags & FRF_USEIP) ? AF_INET : AF_COIP */
    sip->sin_family = AF_INET;	/* Use INET routing for COIP family */
    sip->sin_addr.s_addr = routeto;


    /* Lookup route */
    rtalloc ( &( path ) );
    rep = path.ro_rt;
    if ( rep EQ (struct rtentry *) NULL ) /* Not reachable */
    {
	IfLog (DBG_SKT_RUT)
	    (void) printf ( "No route to %s target 0x%x\n",
			    (flags & FRF_USEIP) ? "IP" : "ST", routeto );
	BUGSTOP (-,routeto,0xa0301);
	arcp->error = NoRouteToDest;
	arcp->flags |= RouteFailed | RouteNoRoute;
	arclp->nxtfree = 0;
	return;
    }


    /* Save local address of outgoing interface. */
    localIPAdr = st2_LocalIPAdrOfGifc( Map2Gifc (rep->rt_ifp) );
    if ( localIPAdr EQ INADDR_ANY )
	BUGSTOP (-,Inconsistency,0xa0302);

    /* ============================================================ */
    /* 2 cases: local delivery (TO_APPL) and non-local (TO_GATEWAY/TO_HOST) */

    if (    (localIPAdr EQ routeto)	/* ???  */
	AND ((flags & FRF_NOSKT) EQ 0) ) /* not to socket if have a srcrut */
    {
	arcp->type       = TO_APPL;
	arcp->nxtSTIPAdr = localIPAdr;
	arcp->nxtIPAdr   = localIPAdr;
	arcp->fwdtoIPAdr = localIPAdr;
	arcp->lclIPAdr   = localIPAdr;	/* EQ nextst EQ routeto */
	arcp->order      = 1;
	arclp->nxtfree   = 1;		/* only 1 choice in BSD */
    }

    /* ============================================================ */
    else				/* Non-local next hop */
    {
	if ( (rep->rt_flags & RTF_GATEWAY) NE 0 )
	{				/* dest is reached via a gateway */
	    arcp->type = TO_GATEWAY;
	    sip = (struct sockaddr_in *) &( rep->rt_gateway );

	    IfLog (DBG_SKT_RUT)
		(void) printf ( "Route to %x via gateway %x, flags %x\n",
			        routeto, sip->sin_addr.s_addr,
			        rep->rt_flags );
	}
	else
	{				/* dest is directly connected */
	    arcp->type = TO_HOST;
	    sip = (struct sockaddr_in *) &( path.ro_dst );

	    IfLog (DBG_SKT_RUT)
		(void) printf ( "Route directly to %x, flags %x\n",
			        routeto, sip->sin_addr.s_addr,
			        rep->rt_flags );
	}

	arcp->gifcp      = Map2Gifc (rep->rt_ifp);

	/* Assuming no srcrut ... */	/* (Assume ST everywhere :-) */
	arcp->nxtSTIPAdr = ((flags & FRF_USEIP) NE 0)
			   ? nextst : sip->sin_addr.s_addr;
	arcp->nxtIPAdr   = sip->sin_addr.s_addr;

	arcp->fwdtoIPAdr = sip->sin_addr.s_addr;
	arcp->lclIPAdr   = localIPAdr;
	arcp->order      = 1;

	/* check that returned route meets strict ST srcrut */
	if ( ((flags & FRF_DIRECT) NE 0) AND (arcp->nxtIPAdr NE routeto) )
	{
	    arcp->error = SourceRouteError;
	    arcp->flags |= RouteFailed | RouteNoRoute;
	}

	if (   ((rep->rt_flags & RTFX_ENCAP) NE 0)
	    OR ((flags & FRF_USEIP) NE 0) )
	{
	    arcp->IPencapsulate = 1;
	    arcp->TOS = 0x38;	/* Priority (1), Low Delay, High Throughput */
	    arcp->TTL = 60;
	}

	if ( (rep->rt_flags & RTFX_SRCRUT) NE 0 )
	{
	    arcp->rtblsrlen = RTEGetSrcRutLen (rep);
	    arcp->rtblsrp = RTEGetaSrcRutp (rep); /* May be multiple */
	    switch ( ntoh1 (arcp->rtblsrp->pcode) )
	    {
	    case STpIPLSrcRut:		/* 16 */
	    case STpIPSSrcRut:		/* 17 */
		arcp->IPencapsulate = 1;
		arcp->TOS = 0x38; /* Priority (1), Low Delay, High Throughput */
		arcp->TTL = 60;
		arcp->nxtSTIPAdr = ntohIP (arcp->rtblsrp->IPAdr[
				(  ntoh1 (arcp->rtblsrp->plen)
				 - sizeof (struct aSrcRut))      / 4]);
		arcp->nxtIPAdr   = ntohIP (arcp->rtblsrp->IPAdr[ 0 ]);
		break;

	    case STpSTLSrcRut:		/* 18 */
	    case STpSTSSrcRut:		/* 19 */
					/* no successive IP routes ... */
		arcp->nxtSTIPAdr = ntohIP (arcp->rtblsrp->IPAdr[0]);
		arcp->nxtIPAdr   = arcp->nxtSTIPAdr;
		break;
	    } /* end of switch */

	    /* ??? Verify type of last srcrut NE first of targetp->targsrp */
	}
	arclp->nxtfree = 1;		/* only 1 choice in BSD */
    }

    /* ============================================================ */

    rtfree ( rep );

    return;
}

/*
 Local Variables:
 c-indent-level: 4
 c-brace-offset: -4
 c-brace-imaginary-offset: 0
 c-continued-statement-offset: 4
 comment-column: 40
 End:
*/
